Imports System.Data.SqlClient

'Autor: Roy Rojas
'email: info@dotnetcr.com
'Fecha: 16/Julio/2006
'Descripcin: Creamos un sistema de conexion a la bd que nos permite 
'               invocar procedimientos almacenados de una forma muy sencilla
'Licencia: Gratuita para fines educativos. 

Public Class Conexion

#Region " Variables "

    Private mUsuario As String
    Private mPassword As String
    Private mConSSPI As Boolean = False
    Private mServidor As String
    Private mBaseDatos As String
    Private mSqlConn As SqlConnection

#End Region

#Region " Propiedades "

    Public Property Usuario() As String
        Get
            Return mUsuario
        End Get
        Set(ByVal Value As String)
            mUsuario = Value
        End Set
    End Property

    Public Property Password() As String
        Get
            Return mPassword
        End Get
        Set(ByVal Value As String)
            mPassword = Value
        End Set
    End Property

    Public Property ConSSPI() As Boolean
        Get
            Return mConSSPI
        End Get
        Set(ByVal Value As Boolean)
            mConSSPI = Value
        End Set
    End Property

    Public Property Servidor() As String
        Get
            Return mServidor
        End Get
        Set(ByVal Value As String)
            mServidor = Value
        End Set
    End Property

    Public Property BaseDatos() As String
        Get
            Return mBaseDatos
        End Get
        Set(ByVal Value As String)
            mBaseDatos = Value
        End Set
    End Property

    Public Property SQLConn() As SqlConnection
        Get
            Return mSqlConn
        End Get
        Set(ByVal Value As SqlConnection)
            mSqlConn = Value
        End Set
    End Property

#End Region

    'Crea el string de conexion a la base de datos.
    Private Function StrConexion() As String
        Try
            Dim strConn As String
            strConn = "Server=" & Servidor & "; " & _
                        "DataBase=" & BaseDatos & "; "
            If Not ConSSPI Then
                strConn &= "user id=" & Usuario & ";password=" & Password
            Else
                strConn &= "Integrated Security=SSPI"
            End If

            Return strConn
        Catch ex As Exception
            Throw ex
        End Try
    End Function

    'Consulta simple a la base de datos, utilizando un query directo.
    Public Function ConsultaBD(ByVal pQuery As String) As DataSet
        Try
            Return CreateDataSet(pQuery)
        Catch ex As Exception
            Throw ex
        End Try
    End Function

    Private Function CreateDataSet(ByVal strSQL As String) As DataSet
        Try
            'Se crea la conexion a la base de datos.
            Dim sqlConn As New SqlConnection(Me.StrConexion)

            'SqlCommand es utilizado para ejecutar los comandos SQL
            Dim sqlCmd As New SqlCommand(strSQL, sqlConn)

            'Se le define el tiempo de espera en segundos para la consulta,
            'el valor default es 30 segundos.
            sqlCmd.CommandTimeout = 3600

            'SqlAdapter utiliza el SqlCommand para llenar el Dataset
            Dim sda As New SqlDataAdapter(sqlCmd)

            'Se llena el dataset
            Dim ds As New DataSet
            sda.Fill(ds)

            Return ds
        Catch ex As Exception
            Throw ex
        End Try
    End Function

    Public Sub New()
        Try
            'Esta seccion de variables deberia ser leida 
            'de un archivo de configuracion .Config
            Me.Servidor = "star"
            Me.BaseDatos = "SIB_NET"
            Me.Usuario = "DIMSA"
            Me.Password = "DIMSA"

            SQLConn = New SqlConnection(Me.StrConexion)
        Catch ex As Exception
            Throw ex
        End Try
    End Sub

End Class

Public Class StoredProcedure

#Region " Variables "
    Private mNombreProcedimiento As String
    Private mParametros As Collection
#End Region

#Region " Propiedades "
    Public Property Nombre() As String
        Get
            Return mNombreProcedimiento
        End Get
        Set(ByVal Value As String)
            mNombreProcedimiento = Value
        End Set
    End Property

    Public Property Parametros() As Collection
        Get
            Return mParametros
        End Get
        Set(ByVal Value As Collection)
            mParametros = Value
        End Set
    End Property
#End Region

#Region " Constructor "

    'Solo recibe el nombre del procedimiento e inicializa la coleccin.
    Public Sub New(ByVal nNombre As String)
        Try
            Nombre = nNombre
            Parametros = New Collection
        Catch ex As Exception
            Throw ex
        End Try
    End Sub

#End Region

#Region " Propiedades y Funciones "

    'Agrega los parametros del procedimiento y su respectivo valor.
    Public Sub AgregarParametro(ByVal pVariable As String, ByVal pValor As Object)
        Try
            Dim iParametro As New StoredProcedureParameter("@" & pVariable, pValor)
            Me.Parametros.Add(iParametro)
        Catch ex As Exception
            Throw ex
        End Try
    End Sub

    'Ejecuta el procedimiento almacenado.
    Public Function EjecutarProcedimiento() As DataSet
        Try
            Dim Conn As New Conexion
            Dim sqlCmd As New SqlCommand(Me.Nombre, Conn.SQLConn)
            sqlCmd.CommandType = CommandType.StoredProcedure

            Dim mParametro As StoredProcedureParameter

            'Agrega las variables al procedimiento almacenado
            For Each mParametro In Me.Parametros
                Dim pParam As New SqlParameter(mParametro.Variable, mParametro.GetTypeProperty)
                pParam.Direction = ParameterDirection.Input
                pParam.Value = mParametro.Valor
                sqlCmd.Parameters.Add(pParam)
            Next

            'SqlAdapter utiliza el SqlCommand para llenar el Dataset
            Dim sda As New SqlDataAdapter(sqlCmd)

            'Se llena el dataset
            Dim ds As New DataSet
            sda.Fill(ds)

            Conn.SQLConn.Close()
            Return ds
        Catch ex As Exception
            Throw ex
        End Try
    End Function

#End Region

End Class

Public Class StoredProcedureParameter

    Private mVariable As String
    Private mValor As Object

    'Nombre de la variable, debe ser igual a la declarada en el procedimiento almacenado
    Public Property Variable() As String
        Get
            Return mVariable
        End Get
        Set(ByVal Value As String)
            mVariable = Value
        End Set
    End Property

    'Valor de la variable, puede ser de cualquier tipo de dato. preferible que 
    'coincida con las variables declaradas en GetTypeProperty
    Public Property Valor()
        Get
            Return mValor
        End Get
        Set(ByVal Value)
            mValor = Value
        End Set
    End Property

    'Se definen los posibles tipos de datos que se le van a enviar al procedimiento almacenado
    'Esta lista podria aumentar conforme se usen otro tipo de variable.
    Public ReadOnly Property GetTypeProperty() As SqlDbType
        Get
            If mValor.GetType.FullName = "System.String" Then
                Return SqlDbType.VarChar
            ElseIf mValor.GetType.FullName = "System.Int16" Then
                Return SqlDbType.Int
            ElseIf mValor.GetType.FullName = "System.Int32" Then
                Return SqlDbType.Int
            ElseIf mValor.GetType.FullName = "System.Int64" Then
                Return SqlDbType.Int
            ElseIf mValor.GetType.FullName = "System.Decimal" Then
                Return SqlDbType.Decimal
            ElseIf mValor.GetType.FullName = "System.Double" Then
                Return SqlDbType.BigInt
            ElseIf mValor.GetType.FullName = "System.DateTime" Then
                Return SqlDbType.DateTime
            ElseIf mValor.GetType.FullName = "System.Byte" Then
                Return SqlDbType.Image
            End If
        End Get
    End Property

    'Procedimiento de creacion de la variable.
    Public Sub New(ByVal pVariable As String, ByVal pValor As Object)
        Try
            Me.Variable = pVariable
            Me.Valor = pValor
        Catch ex As Exception
            Throw New Exception("Error en la creacion del Parametro" & vbCrLf & ex.Message)
        End Try
    End Sub

End Class
